@@ -94,40 +94,40 @@
 #'             size = 3) +
 #'   theme(legend.position = "none")
 #'
-position_dodgenudge <- function(width = 1,
-                                preserve = c("total", "single"),
-                                x = 0,
-                                y = 0,
-                                direction = "none",
-                                kept.origin = "dodged") {
-  # Ensure error message is triggered early
-  if (!kept.origin %in% c("original", "dodged", "none")) {
-    stop("Invalid 'kept.origin': ", kept.origin,
-         "expected: `\"original\", \"dodged\" or \"none\"")
-  }
+position_dodgenudge <-
+  function(width = 1,
+           preserve = c("total", "single"),
+           x = 0,
+           y = 0,
+           direction = c("none", "split", "split.x", "split.y", "center"),
+           kept.origin = c("dodged", "original", "none")) {
 
-  ggplot2::ggproto(NULL, PositionDodgeAndNudge,
-                   x = x,
-                   y = y,
-                   .fun_x = switch(direction,
-                                   none = function(x) {1},
-                                   split = sign,
-                                   split.y = function(x) {1},
-                                   split.x = sign,
-                                   center = sign,
-                                   function(x) {1}),
-                   .fun_y = switch(direction,
-                                   none = function(x) {1},
-                                   split = sign,
-                                   split.x = function(x) {1},
-                                   split.y = sign,
-                                   center = sign,
-                                   function(x) {1}),
-                   kept.origin = kept.origin,
-                   width = width,
-                   preserve = match.arg(preserve)
-  )
-}
+    preserve <- rlang::arg_match(preserve)
+    direction <- rlang::arg_match(direction)
+    kept.origin <- rlang::arg_match(kept.origin)
+
+    ggplot2::ggproto(NULL, PositionDodgeAndNudge,
+                     x = x,
+                     y = y,
+                     .fun_x = switch(direction,
+                                     none = function(x) {1},
+                                     split = sign,
+                                     split.y = function(x) {1},
+                                     split.x = sign,
+                                     center = sign,
+                                     function(x) {1}),
+                     .fun_y = switch(direction,
+                                     none = function(x) {1},
+                                     split = sign,
+                                     split.x = function(x) {1},
+                                     split.y = sign,
+                                     center = sign,
+                                     function(x) {1}),
+                     kept.origin = kept.origin,
+                     width = width,
+                     preserve = rlang::arg_match(preserve)
+    )
+  }
 
 #' @rdname ggpp-ggproto
 #' @format NULL
@@ -198,7 +198,7 @@ position_dodge_keep <- function(width = 1,
                       preserve = preserve,
                       x = 0,
                       y = 0,
-                      direction = "as.is",
+                      direction = "none",
                       kept.origin = "original")
 }
 
@@ -207,11 +207,11 @@ position_dodge_keep <- function(width = 1,
 #' @export
 #'
 position_dodge2_keep <- function(width = 1,
-                                preserve = c("total", "single")) {
+                                 preserve = c("total", "single")) {
   position_dodge2nudge(width = width,
-                      preserve = preserve,
-                      x = 0,
-                      y = 0,
-                      direction = "as.is",
-                      kept.origin = "original")
+                       preserve = preserve,
+                       x = 0,
+                       y = 0,
+                       direction = "none",
+                       kept.origin = "original")
 }
