@@ -39,6 +39,24 @@
 #'   observations selected, whenever \code{keep.fraction} results in fewer
 #'   observations selected, it is obeyed.
 #'
+#'   Computation of density and of the default bandwidth require at least
+#'   two observations with different values. If data do not fulfill this
+#'   condition, they are kept only if \code{keep.fraction = 1}. This is correct
+#'   behavior for a single observation, but can be surprising in the case of
+#'   multiple observations.
+#'
+#'   Parameters \code{keep.these} and \code{exclude.these} make it possible to
+#'   force inclusion or exclusion of observations after the density is computed.
+#'   In case of conflict, \code{exclude.these} overrides \code{keep.these}.
+#'
+#' @note Which points are kept and which not depends on how dense a grid is used
+#'   and how flexible the density surface estimate is. This depends on the
+#'   values passed as arguments to parameters \code{n}, \code{bw} and
+#'   \code{kernel}. It is also important to be aware that both
+#'   \code{geom_text()} and \code{geom_text_repel()} can avoid overplotting by
+#'   discarding labels at the plot rendering stage, i.e., what is plotted may
+#'   differ from what is returned by this statistic.
+#'
 #' @param mapping The aesthetic mapping, usually constructed with
 #'   \code{\link[ggplot2]{aes}} or \code{\link[ggplot2]{aes_}}. Only needs
 #'   to be set at the layer level if you are overriding the plot defaults.
@@ -53,10 +71,15 @@
 #' @param keep.sparse logical If \code{TRUE}, the default, observations from the
 #'   more sparse regions are retained, if \code{FALSE} those from the densest
 #'   regions.
-#' @param keep.these character vector, integer vector, logical vector or
-#'   function that takes the variable mapped to the \code{label} aesthetic as
-#'   first argument and returns a character vector or a logical vector. These
-#'   rows from \code{data} are selected irrespective of the local density.
+#' @param keep.these,exclude.these character vector, integer vector, logical
+#'   vector or function that takes one or more variables in data selected by
+#'   \code{these.target}. Negative integers behave as in R's extraction methods.
+#'   The rows from \code{data} indicated by \code{keep.these} and
+#'   \code{exclude.these} are kept or excluded irrespective of the local
+#'   density.
+#' @param these.target character, numeric or logical selecting one or more
+#'   column(s) of \code{data}. If \code{TRUE} the whole \code{data} object is
+#'   passed.
 #' @param pool.along character, one of \code{"none"} or \code{"x"},
 #'   indicating if selection should be done pooling the observations along the
 #'   \emph{x} aesthetic, or separately on either side of \code{xintercept}.
@@ -199,7 +222,9 @@ stat_dens2d_labels <-
            keep.number = Inf,
            keep.sparse = TRUE,
            keep.these = FALSE,
-           pool.along = "xy",
+           exclude.these = FALSE,
+           these.target = "label",
+           pool.along = c("xy", "x", "y", "none"),
            xintercept = 0,
            yintercept = 0,
            invert.selection = FALSE,
@@ -211,6 +236,8 @@ stat_dens2d_labels <-
            show.legend = FALSE,
            inherit.aes = TRUE) {
 
+    pool.along <- rlang::arg_match(pool.along)
+
     if (length(label.fill) > 1L) {
       stop("Length for 'label.fill' is not 1: ", label.fill)
     }
@@ -247,6 +274,8 @@ stat_dens2d_labels <-
                     keep.number = keep.number,
                     keep.sparse = keep.sparse,
                     keep.these = keep.these,
+                    exclude.these = exclude.these,
+                    these.target = these.target,
                     pool.along = pool.along,
                     xintercept = xintercept,
                     yintercept = yintercept,
@@ -259,174 +288,6 @@ stat_dens2d_labels <-
     )
   }
 
-dens2d_labs_compute_fun <-
-  function(data,
-           scales,
-           keep.fraction,
-           keep.number,
-           keep.sparse,
-           keep.these,
-           pool.along,
-           xintercept,
-           yintercept,
-           invert.selection,
-           h,
-           n,
-           label.fill,
-           return.density) {
-
-    force(data)
-    if (!exists("label", data) && !is.null(label.fill)) {
-      data[["label"]] <- rownames(data)
-    }
-
-    keep.these <- keep_these2logical(keep.these = keep.these, data = data)
-
-    # discard redundant splits
-    if (pool.along != "xy") {
-      if (pool.along == "y" &&
-          !(xintercept < max(data[["x"]]) &&
-            xintercept > min(data[["x"]]))) {
-        pool.along <- "xy"
-      } else if (pool.along == "x" &&
-                 !(yintercept < max(data[["y"]]) &&
-                   yintercept > min(data[["y"]]))) {
-        pool.along <- "xy"
-      } else if (pool.along == "none") {
-        if (!(xintercept < max(data[["x"]]) &&
-              xintercept > min(data[["x"]])) &&
-            !(yintercept < max(data[["y"]]) &&
-              yintercept > min(data[["y"]]))) {
-          pool.along <- "xy"
-        } else if (!(xintercept < max(data[["x"]]) &&
-                     xintercept > min(data[["x"]]))) {
-          pool.along <- "x"
-        } else if (!(yintercept < max(data[["y"]]) &&
-                     yintercept > min(data[["y"]]))) {
-          pool.along <- "y"
-        }
-      }
-    }
-
-    # make list of logical vectors
-    if (pool.along == "y") {
-       selectors <-list(q12 = data[["x"]] <= xintercept,
-                        q34 = data[["x"]] > xintercept)
-      if (length(keep.fraction) != 2L) {
-        keep.fraction <- rep_len(keep.fraction, length.out = 2)
-      }
-      if (length(keep.number) != 2L) {
-        if (length(keep.number) == 1L) {
-          keep.number <- keep.number %/% 2
-        }
-        keep.number <- rep_len(keep.number, length.out = 2)
-      }
-      num.rows <- sapply(selectors, sum) # selectors are logical
-    } else if (pool.along == "x") {
-      selectors <-list(q23 = data[["y"]] <= yintercept,
-                       q41 = data[["y"]] > yintercept)
-      if (length(keep.fraction) != 2L) {
-        keep.fraction <- rep_len(keep.fraction, length.out = 2)
-      }
-      if (length(keep.number) != 2L) {
-        if (length(keep.number) == 1L) {
-          keep.number <- keep.number %/% 2
-        }
-        keep.number <- rep_len(keep.number, length.out = 2)
-      }
-      num.rows <- sapply(selectors, sum) # selectors are logical
-    } else if (pool.along == "none") {
-      selectors <-list(q1 = data[["y"]] >= yintercept & data[["x"]] >= xintercept,
-                       q2 = data[["y"]] < yintercept & data[["x"]] >= xintercept,
-                       q3 = data[["y"]] < yintercept & data[["x"]] < xintercept,
-                       q4 = data[["y"]] > yintercept & data[["x"]] < xintercept)
-      if (length(keep.fraction) != 4L) {
-        keep.fraction <- rep_len(keep.fraction, length.out = 4)
-      }
-      if (length(keep.number) != 4L) {
-        if (length(keep.number) == 1L) {
-          keep.number <- keep.number %/% 4
-        }
-        keep.number <- rep_len(keep.number, length.out = 4)
-      }
-      num.rows <- sapply(selectors, sum) # selectors are logical
-    } else {
-      keep.fraction <- keep.fraction[[1]] # can be a vector or a list
-      keep.number <- keep.number[[1]]
-      num.rows <- nrow(data)
-      selectors <- list(all = rep.int(TRUE, times = num.rows))
-    }
-
-    # vectorized
-    too.large.frac <- num.rows * keep.fraction > keep.number
-    keep.fraction[too.large.frac] <-
-      keep.number[too.large.frac] / num.rows[too.large.frac]
-
-    # estimate 2D density
-    if (is.null(h)) {
-      h <- c(MASS::bandwidth.nrd(data$x), MASS::bandwidth.nrd(data$y))
-    }
-    if (is.null(n)) {
-      n <- trunc(sqrt(nrow(data))) * 8L
-    }
-    kk <-  MASS::kde2d(
-      data[["x"]], data[["y"]], h = h, n = n,
-      lims = c(scales$x$dimension(), scales$y$dimension()))
-    dimnames(kk[["z"]]) <- list(kk[["x"]], kk[["y"]])
-
-    # compute 2D density at each onservations coordinates
-    kx <- cut(data$x, kk$x, labels = FALSE, include.lowest = TRUE)
-    ky <- cut(data$y, kk$y, labels = FALSE, include.lowest = TRUE)
-    kz <- sapply(seq_along(kx), function(i) kk$z[kx[i], ky[i]])
-
-    # we construct one logical vector by adding observations/label to be kept
-    # we may have a list of 1, 2, or 4 logical vectors
-    keep <- keep.these
-    for (i in seq_along(selectors)) {
-      if (keep.fraction[i] == 1) {
-        keep[ selectors[[i]] ] <- TRUE
-      } else if (keep.fraction[i] != 0) {
-        if (keep.sparse) {
-          keep[ selectors[[i]] ] <-
-            kz[ selectors[[i]] ] < stats::quantile(kz[ selectors[[i]] ],
-                                                   keep.fraction[i], names = FALSE)
-        } else {
-          keep[ selectors[[i]] ] <-
-            kz[ selectors[[i]] ] >= stats::quantile(kz[ selectors[[i]] ],
-                                                    1 - keep.fraction[i], names = FALSE)
-        }
-      }
-    }
-
-    if (invert.selection){
-      keep <- !keep
-    }
-
-    if (return.density) {
-      data[["keep.obs"]] <- keep
-      data[["density"]] <- kz
-    }
-
-    if (is.null(label.fill)) {
-      data <- data[keep, ]
-    } else if (is.function(label.fill)) {
-      data[["label"]][!keep] <- label.fill(data[["label"]][!keep])
-    } else if (is.na(label.fill)) {
-      # NA_logical_, the default NA, cannot always be assigned to character
-      label.fill <- NA_character_
-      data[["label"]][!keep] <- label.fill
-    } else if (is.character(label.fill)) {
-      data[["label"]][!keep] <- label.fill
-    } else if (is.logical(label.fill)) {
-      if (label.fill) {
-        data[["label"]][!keep] <- ""
-      } # if FALSE data is not modified
-    } else {
-          stop("'label.fill' is : ", mode(label.fill),
-               " instead of 'character' or 'function'.")
-    }
-    data
-}
 
 #' @rdname ggpp-ggproto
 #' @format NULL
@@ -437,7 +298,193 @@ StatDens2dLabels <-
     "StatDens2dLabels",
     ggplot2::Stat,
     compute_panel =
-      dens2d_labs_compute_fun,
+      function(data,
+               scales,
+               keep.fraction,
+               keep.number,
+               keep.sparse,
+               keep.these,
+               exclude.these,
+               these.target,
+               pool.along,
+               xintercept,
+               yintercept,
+               invert.selection,
+               h,
+               n,
+               label.fill,
+               return.density) {
+
+        force(data)
+        if (!exists("label", data) && !is.null(label.fill)) {
+          data[["label"]] <- rownames(data)
+        }
+
+        keep.these <- these2logical(these = keep.these,
+                                    data = data,
+                                    these.target = these.target)
+
+        exclude.these <- these2logical(these = exclude.these,
+                                       data = data,
+                                       these.target = these.target)
+
+        # discard redundant splits
+        if (pool.along != "xy") {
+          if (pool.along == "y" &&
+              !(xintercept < max(data[["x"]]) &&
+                xintercept > min(data[["x"]]))) {
+            pool.along <- "xy"
+          } else if (pool.along == "x" &&
+                     !(yintercept < max(data[["y"]]) &&
+                       yintercept > min(data[["y"]]))) {
+            pool.along <- "xy"
+          } else if (pool.along == "none") {
+            if (!(xintercept < max(data[["x"]]) &&
+                  xintercept > min(data[["x"]])) &&
+                !(yintercept < max(data[["y"]]) &&
+                  yintercept > min(data[["y"]]))) {
+              pool.along <- "xy"
+            } else if (!(xintercept < max(data[["x"]]) &&
+                         xintercept > min(data[["x"]]))) {
+              pool.along <- "x"
+            } else if (!(yintercept < max(data[["y"]]) &&
+                         yintercept > min(data[["y"]]))) {
+              pool.along <- "y"
+            }
+          }
+        }
+
+        # make list of logical vectors
+        if (pool.along == "y") {
+          selectors <-list(q12 = data[["x"]] <= xintercept,
+                           q34 = data[["x"]] > xintercept)
+          if (length(keep.fraction) != 2L) {
+            keep.fraction <- rep_len(keep.fraction, length.out = 2)
+          }
+          if (length(keep.number) != 2L) {
+            if (length(keep.number) == 1L) {
+              keep.number <- keep.number %/% 2
+            }
+            keep.number <- rep_len(keep.number, length.out = 2)
+          }
+          num.rows <- sapply(selectors, sum) # selectors are logical
+        } else if (pool.along == "x") {
+          selectors <-list(q23 = data[["y"]] <= yintercept,
+                           q41 = data[["y"]] > yintercept)
+          if (length(keep.fraction) != 2L) {
+            keep.fraction <- rep_len(keep.fraction, length.out = 2)
+          }
+          if (length(keep.number) != 2L) {
+            if (length(keep.number) == 1L) {
+              keep.number <- keep.number %/% 2
+            }
+            keep.number <- rep_len(keep.number, length.out = 2)
+          }
+          num.rows <- sapply(selectors, sum) # selectors are logical
+        } else if (pool.along == "none") {
+          selectors <-list(q1 = data[["y"]] >= yintercept & data[["x"]] >= xintercept,
+                           q2 = data[["y"]] < yintercept & data[["x"]] >= xintercept,
+                           q3 = data[["y"]] < yintercept & data[["x"]] < xintercept,
+                           q4 = data[["y"]] > yintercept & data[["x"]] < xintercept)
+          if (length(keep.fraction) != 4L) {
+            keep.fraction <- rep_len(keep.fraction, length.out = 4)
+          }
+          if (length(keep.number) != 4L) {
+            if (length(keep.number) == 1L) {
+              keep.number <- keep.number %/% 4
+            }
+            keep.number <- rep_len(keep.number, length.out = 4)
+          }
+          num.rows <- sapply(selectors, sum) # selectors are logical
+        } else {
+          keep.fraction <- keep.fraction[[1]] # can be a vector or a list
+          keep.number <- keep.number[[1]]
+          num.rows <- nrow(data)
+          selectors <- list(all = rep.int(TRUE, times = num.rows))
+        }
+
+        # vectorized
+        too.large.frac <- num.rows * keep.fraction > keep.number
+        keep.fraction[too.large.frac] <-
+          keep.number[too.large.frac] / num.rows[too.large.frac]
+
+        # estimate 2D density
+        # data with fewer than 2 distinct values preventgs density() estimation
+        if (length(unique(data$x)) >= 2L &&
+            length(unique(data$y)) >= 2L) {
+          if (is.null(h)) {
+            h <- c(MASS::bandwidth.nrd(data$x), MASS::bandwidth.nrd(data$y))
+          }
+          if (is.null(n)) {
+            n <- trunc(sqrt(nrow(data))) * 8L
+          }
+          kk <-  MASS::kde2d(
+            data[["x"]], data[["y"]], h = h, n = n,
+            lims = c(scales$x$dimension(), scales$y$dimension()))
+          dimnames(kk[["z"]]) <- list(kk[["x"]], kk[["y"]])
+
+          # compute 2D density at each observation's coordinates
+          kx <- cut(data$x, kk$x, labels = FALSE, include.lowest = TRUE)
+          ky <- cut(data$y, kk$y, labels = FALSE, include.lowest = TRUE)
+          kz <- sapply(seq_along(kx), function(i) kk$z[kx[i], ky[i]])
+        } else {
+          kz <- rep_len(1, nrow(data))
+        }
+
+        # we construct one logical vector by adding observations/label to be kept
+        # we may have a list of 1, 2, or 4 logical vectors
+        keep <- logical(nrow(data))
+        for (i in seq_along(selectors)) {
+          if (keep.fraction[i] == 1) {
+            keep[ selectors[[i]] ] <- TRUE
+          } else if (keep.fraction[i] != 0 && length(selectors[[i]]) >= 2L) {
+            if (keep.sparse) {
+              keep[ selectors[[i]] ] <-
+                kz[ selectors[[i]] ] < stats::quantile(kz[ selectors[[i]] ],
+                                                       keep.fraction[i],
+                                                       names = FALSE,
+                                                       type = 8)
+            } else {
+              keep[ selectors[[i]] ] <-
+                kz[ selectors[[i]] ] >= stats::quantile(kz[ selectors[[i]] ],
+                                                        1 - keep.fraction[i],
+                                                        names = FALSE,
+                                                        type = 8)
+            }
+          }
+        }
+        keep <- (keep | keep.these) & !exclude.these
+
+        if (invert.selection){
+          keep <- !keep
+        }
+
+        if (return.density) {
+          data[["keep.obs"]] <- keep
+          data[["density"]] <- kz
+        }
+
+        if (is.null(label.fill)) {
+          data <- data[keep, ]
+        } else if (is.function(label.fill)) {
+          data[["label"]][!keep] <- label.fill(data[["label"]][!keep])
+        } else if (is.na(label.fill)) {
+          # NA_logical_, the default NA, cannot always be assigned to character
+          label.fill <- NA_character_
+          data[["label"]][!keep] <- label.fill
+        } else if (is.character(label.fill)) {
+          data[["label"]][!keep] <- label.fill
+        } else if (is.logical(label.fill)) {
+          if (label.fill) {
+            data[["label"]][!keep] <- ""
+          } # if FALSE data is not modified
+        } else {
+          stop("'label.fill' is : ", mode(label.fill),
+               " instead of 'character' or 'function'.")
+        }
+        data
+      },
+
     required_aes = c("x", "y")
   )
 
@@ -454,32 +501,50 @@ StatDens2dLabels <-
 #'
 #' @keywords internal
 #'
-keep_these2logical <- function(keep.these, data) {
-  if (length(keep.these)) {
-    if (is.function(keep.these)) {
-      keep.these <- keep.these(data$label) # character or logical vector
-    }
-    if (is.character(keep.these)) {
-      keep.these <- data$label %in% keep.these # logical vector
-    }
-    if (is.numeric(keep.these)) { # positional indices
-      temp <- rep(FALSE, times = nrow(data))
-      temp[keep.these] <- TRUE
-      keep.these <- temp
-    }
-    if (is.logical(keep.these)) { # logical indices, if short recycle
-      if (length(keep.these) >= 1L && length(keep.these) < nrow(data)) {
-        keep.these <- rep(keep.these, length.out = nrow(data))
-      } else if (length(keep.these) > nrow(data)) {
-        stop("Logical vector 'keep.these' longer than data")
+these2logical<- function(these,
+                         data,
+                         these.target = "label") {
+  if (length(these)) {
+    if (is.character(these) || is.function(these)) {
+      if (is.character(these.target) && any(these.target == "label") &&
+          !exists("label", where = data, mode = "character", inherits = FALSE)) {
+        data$label <- rownames(data)
+      }
+      if (is.character(these.target)) {
+        orig.num.targets <- length(unique(these.target))
+        these.target <- intersect(these.target, colnames(data))
+        if (length(these.target) == 0L) {
+          stop("Variables in 'these.target' not in 'data'")
+        } else if (orig.num.targets > length(these.target)) {
+          warning("Some variables in 'these.target' not in 'data'")
+        }
       }
     }
-    if (anyNA(keep.these)) {
-      warning("Discarding 'NA's in keep.these")
-      keep.these[is.na(keep.these)] <- FALSE
+    if (is.function(these)) {
+      these <- these(data[ , these.target, drop = TRUE]) # any vector
+    }
+    if (is.character(these)) {
+      stopifnot(is.character(data[[these.target[1]]]))
+      these <- data[[these.target[1]]] %in% these # logical vector
+    }
+    if (is.numeric(these)) { # positional indices
+      temp <- rep_len(FALSE, length.out = nrow(data))
+      temp[these] <- TRUE
+      these <- temp
+    }
+    if (is.logical(these)) { # logical indices, if short recycle
+      if (length(these) >= 1L && length(these) < nrow(data)) {
+        these <- rep_len(these, length.out = nrow(data))
+      } else if (length(these) > nrow(data)) {
+        stop("Logical vector 'keep.these' or 'exclude.these' longer than data")
+      }
+    }
+    if (anyNA(these)) {
+      warning("Discarding 'NA's in 'keep.these' or 'exclude-these'")
+      these[is.na(these)] <- FALSE
     }
   } else { # replace NULL and vectors with length zero with FALSE
-    keep.these <- rep(FALSE, nrow(data))
+    these <- rep_len(FALSE, length.out = nrow(data))
   }
-  keep.these
+  these
 }
